import json
import os

import gradio as gr
import yaml

from .translator import Translator


css = """
.form {
  padding: 0px;
  border: none;
}
textarea::placeholder {
  color: rgb(100,100,100)
}
textarea[placeholder="Translation"] {
  background-color: rgb(248,249,250);
}
"""

# See https://github.com/gradio-app/gradio/issues/6101.
shortcut_js = """
<script>
document.addEventListener("keydown", function(e) {
  if (e.key == "Enter" && (e.ctrlKey || e.shiftKey || e.metaKey)) {
    document.getElementById("translate_btn").click();
  }
})
</script>
"""


def load_settings(filename: str) -> tuple[list, list]:
    with open(filename, "r", encoding="utf-8") as f:
        settings = yaml.safe_load(f)

    llm_settings = settings.get("llms")
    if not llm_settings:
        raise gr.Error("No LLMs found in settings.yaml", duration=5)

    language_settings = settings.get("languages")
    if not language_settings:
        raise gr.Error("No languges found in settings.yaml", duration=5)

    llms = []
    for llm in llm_settings:
        title = llm.pop("title", "")
        api_key = llm.get("api_key")
        if api_key.startswith("${") and api_key.endswith("}"):
            # Replace the environment variable with its value.
            var = api_key[2:-1]
            value = os.getenv(var, "")
            llm["api_key"] = value
        llms.append((title, json.dumps(llm)))

    languages = [
        ("Auto Detect", "auto"),
    ]
    for lang in language_settings:
        languages.append((lang["title"], lang["language"]))

    return llms, languages


class App:
    def __init__(self, llms: list, languages: list) -> None:
        self.LLMS: list = llms
        self.LANGUAGES: list = languages
        self.demo: gr.Blocks | None = None

    def build(self, translator: Translator) -> None:
        with gr.Blocks(title="LLM Translate", css=css, head=shortcut_js) as demo:
            self.demo = demo

            with gr.Sidebar():
                with gr.Tab("Settings"):
                    llm = gr.Dropdown(
                        label="LLM",
                        choices=self.LLMS,
                        value=self.LLMS[0][1],
                    )
                    llm.select(
                        fn=translator.switch_llm,
                        inputs=llm,
                    )

            gr.Markdown("# 🌐 LLM Translate")

            with gr.Row():
                with gr.Column():
                    source_lang = gr.Dropdown(
                        choices=self.LANGUAGES,
                        value="auto",
                        show_label=False,
                    )
                    input_text = gr.Textbox(
                        label="",
                        placeholder="Enter text",
                        lines=10,
                        max_lines=50,
                        autofocus=True,
                    )
                with gr.Column():
                    target_lang = gr.Dropdown(
                        choices=self.LANGUAGES[1:],  # Exclude "auto" option
                        value=self.LANGUAGES[1][1],  # The first language after "auto"
                        show_label=False,
                    )
                    output_text = gr.Textbox(
                        label="",
                        placeholder="Translation",
                        lines=10,
                        max_lines=50,
                        interactive=False,
                        show_copy_button=True,
                    )

            input_text.input(
                fn=translator.detect,
                inputs=[input_text],
                outputs=source_lang,
                show_progress="hidden",
            )

            translate_btn = gr.Button(
                "Translate (Enter + Ctrl/Shift/Cmd)",
                variant="primary",
                elem_id="translate_btn",
            )
            translate_btn.click(
                fn=translator.translate,
                inputs=[input_text, source_lang, target_lang],
                outputs=output_text,
                show_progress="minimal",
            )

            # Initialize app when page loads
            demo.load(translator.initialize, inputs=llm)
            # Clean up app when page is closed
            demo.unload(translator.cleanup)

    def run(
        self, server_port: int = 7860, quiet: bool = False, inbrowser: bool = True
    ) -> None:
        self.demo.launch(server_port=server_port, quiet=quiet, inbrowser=inbrowser)



import json
from typing import AsyncIterator

from coagent.agents import ChatAgent, ChatMessage, ModelClient
from coagent.agents.structured_agent import StructuredAgent
from coagent.core import AgentSpec, Message, new
from coagent.runtimes import LocalRuntime
import gradio as gr


class Input(Message):
    input_text: str
    source_lang: str
    target_lang: str


class Translator:
    def __init__(self, languages: list[str]) -> None:
        self.runtime = LocalRuntime()
        self.started: bool = False
        self.candidate_languages = [lang for _, lang in languages[1:]]  # Exclude "auto"

        langs = "\n".join([f"- {lang}" for lang in self.candidate_languages])
        self.detector = AgentSpec(
            "detector",
            new(
                ChatAgent,
                system=f"""\
You are a language detector who is dedicated to detect the language of the given text.

Your output must be one of the following languages (and nothing else):
{langs}
""",
            ),
        )

        self.agent = AgentSpec(
            "translator",
            new(
                StructuredAgent,
                input_type=Input,
                system="""
    {%- if source_lang == "auto" -%}
    You are a professional translator who is dedicated to translate the given text to {{ target_lang }}.
    {%- else -%}
    You are a professional translator who is dedicated to translate the given text from {{ source_lang }} to {{ target_lang }}.
    {%- endif -%}
    """,
                messages=[
                    ChatMessage(
                        role="user",
                        content="{{ input_text }}",
                    )
                ],
            ),
        )

    async def initialize(self, request: gr.Request, llm: str):
        if self.started:
            return

        await self.runtime.start()
        await self.runtime.register(self.detector)
        await self.runtime.register(self.agent)

        # Switch to the first LLM by default.
        await self.switch_llm(request, llm)

        self.started = True

    async def cleanup(self, request: gr.Request):
        await self.runtime.stop()
        self.started = False

    async def switch_llm(self, request: gr.Request, llm: str):
        settings = json.loads(llm)
        # print(f"Switching LLM to {settings['model']}")

        client = ModelClient(**settings)
        self.detector.constructor.kwargs["client"] = client
        self.agent.constructor.kwargs["client"] = client

    async def detect(self, input_text: str) -> str:
        if not input_text:
            return "auto"

        # print(f'Detecting language for "{input_text}"')
        result = await self.detector.run(
            ChatMessage(role="user", content=input_text).encode(),
        )
        msg = ChatMessage.decode(result)
        detected = msg.content
        # print(f"Detected language: {detected}")

        if detected in self.candidate_languages:
            return detected
        return "auto"  # Fallback to "auto"

    async def translate(
        self, input_text: str, source_lang: str, target_lang: str
    ) -> AsyncIterator[str]:
        if not input_text:
            yield ""
            return

        if source_lang == target_lang:
            yield input_text
            return

        result = await self.agent.run(
            Input(
                input_text=input_text, source_lang=source_lang, target_lang=target_lang
            ).encode(),
            stream=True,
        )

        complete_content = ""
        async for chunk in result:
            msg = ChatMessage.decode(chunk)
            complete_content += msg.content
            yield complete_content



import argparse

from coagent.core import set_stderr_logger

from .app import App, load_settings
from .translator import Translator


def main():
    parser = argparse.ArgumentParser(
        description="Your personal Language Translator powered by LLMs."
    )
    parser.add_argument(
        "settings", type=str, help="The path to the settings file (in YAML)."
    )
    parser.add_argument(
        "--port",
        type=int,
        default=7860,
        help="The port number. (Defaults to %(default)r)",
    )
    parser.add_argument(
        "--level",
        type=str,
        default="INFO",
        help="The logging level. (Defaults to %(default)r)",
    )
    args = parser.parse_args()

    set_stderr_logger(args.level)

    llms, languages = load_settings(args.settings)
    translator = Translator(languages)
    app = App(llms, languages)
    app.build(translator)
    app.run(server_port=args.port, quiet=True, inbrowser=True)


if __name__ == "__main__":
    main()
